--[[
# 显示men,swap,down,up,battery,cpu1,cpu2,使用率。
# 注意：如果 settings_table 第一段落 name 为 cpu 时可能导致错误,
#       从而超过 5 圆环后的数据显示错误或者不显示。为了避免这一
#       情况的发生，请将 name 为 cpu 的段落放置在 settings_table
#       的最后。
# 本脚本修改自著名的 clock_rings.lua
]]

settings_table = {
    -- 内存使用率
    {
        name='memperc',
        arg='',
        max=100,
        bg_colour=0xC4C4C4,
        bg_alpha=0.2,
        fg_colour=0x1EF75E,
        fg_alpha=0.8,
        --[[
	change_color=6,
	start_alpha=0,
	end_alpha=1,
	startcolor=0x0000ff,
	endcolor=0xff0000,
	]]
        x=102, y=195,
        radius=25,
        thickness=4,
        start_angle=0,
        end_angle=360
    },
    -- 交换使用率
    {
        name='swapperc',
        arg='',
        max=100,
        bg_colour=0xC4C4C4,
        bg_alpha=0.2,
        fg_colour=0x1EF75E,
        fg_alpha=0.8,
        --[[
	change_color=6,
	start_alpha=0,
	end_alpha=1,
	startcolor=0x0000ff,
	endcolor=0xff0000,
	]]
        x=102, y=195,
        radius=20,
        thickness=4,
        start_angle=0,
        end_angle=360
    },
    -- 根目录使用率
    {
        name='fs_used_perc',
        arg='/',
        max=100,
        bg_colour=0xC4C4C4,
        bg_alpha=0.2,
        fg_colour=0x1EF75E,
        fg_alpha=0.8,
        --[[
	change_color=6,
	start_alpha=0,
	end_alpha=1,
	startcolor=0x0000ff,
	endcolor=0xff0000,
	]]
        x=40, y=260,
        radius=25,
        thickness=4,
        start_angle=0,
        end_angle=360
    },
    -- 家目录使用率
    {
        name='fs_used_perc',
        arg='/home',
        max=100,
        bg_colour=0xC4C4C4,
        bg_alpha=0.2,
        fg_colour=0x1EF75E,
        fg_alpha=0.8,
        --[[
	change_color=6,
	start_alpha=0,
	end_alpha=1,
	startcolor=0x0000ff,
	endcolor=0xff0000,
	]]
        x=40, y=260,
        radius=20,
        thickness=4,
        start_angle=0,
        end_angle=360
    },
    -- 电池使用率
    {
        name='battery_percent',
        arg='',
        max=100,
        bg_colour=0xC4C4C4,
        bg_alpha=0.2,
        fg_colour=0x1EF75E,
        fg_alpha=0.8,
        --[[
	change_color=6,
	start_alpha=0,
	end_alpha=1,
	startcolor=0x0000ff,
	endcolor=0xff0000,
	]]
        x=102, y=260,
        radius=25,
        thickness=4,
        start_angle=0,
        end_angle=360
    },
    -- cup0 使用率(建议该段落放置在最后)
    {
        name='cpu',
        arg='cpu0',
        max=100,
        bg_colour=0xC4C4C4,
        bg_alpha=0.2,
        fg_colour=0x1EF75E,
        fg_alpha=0.8,
        --[[
	change_color=6,
	start_alpha=0,
	end_alpha=1,
	startcolor=0x0000ff,
	endcolor=0xff0000,
	]]
        x=40, y=195,
        radius=25,
        thickness=4,
        start_angle=0,
        end_angle=360
    },
    -- cup1 使用率(建议该段落放置在最后)
    {
        name='cpu',
        arg='cpu1',
        max=100,
        bg_colour=0xC4C4C4,
        bg_alpha=0.2,
        fg_colour=0x1EF75E,
        fg_alpha=0.8,
        --[[
	change_color=6,
	start_alpha=0,
	end_alpha=1,
	startcolor=0x0000ff,
	endcolor=0xff0000,
	]]
        x=40, y=195,
        radius=20,
        thickness=4,
        start_angle=0,
        end_angle=360
    },
}

-- 需要 cairo 图形库
require 'cairo'

-- rgb颜色
function rgb_to_r_g_b(colour,alpha)
    return ((colour / 0x10000) % 0x100) / 255., ((colour / 0x100) % 0x100) / 255., (colour % 0x100) / 255., alpha
end

-- 绘制圆(常规)
function draw_ring(cr,t,pt)
    local w,h=conky_window.width,conky_window.height

    local xc,yc,ring_r,ring_w,sa,ea=pt['x'],pt['y'],pt['radius'],pt['thickness'],pt['start_angle'],pt['end_angle']
    local bgc, bga, fgc, fga=pt['bg_colour'], pt['bg_alpha'], pt['fg_colour'], pt['fg_alpha']

    local angle_0=sa*(2*math.pi/360)-math.pi/2
    local angle_f=ea*(2*math.pi/360)-math.pi/2
    local t_arc=t*(angle_f-angle_0)

    -- Draw background ring

    cairo_arc(cr,xc,yc,ring_r,angle_0,angle_f)
    cairo_set_source_rgba(cr,rgb_to_r_g_b(bgc,bga))
    cairo_set_line_width(cr,ring_w)
    cairo_stroke(cr)

    -- Draw indicator ring

    cairo_arc(cr,xc,yc,ring_r,angle_0,angle_0+t_arc)
    cairo_set_source_rgba(cr,rgb_to_r_g_b(fgc,fga))
    cairo_stroke(cr)        
end

--[[
-- 绘制圆(可变色)
function draw_ring(cr,t,pt)
    local w,h=conky_window.width,conky_window.height

    local xc,yc,ring_r,ring_w,sa,ea=pt['x'],pt['y'],pt['radius'],pt['thickness'],pt['start_angle'],pt['end_angle']
    local bgc, bga, fgc, fga=pt['bg_colour'], pt['bg_alpha'], pt['fg_colour'], pt['fg_alpha']
    local cr1,t1,pt1=cr,t,pt
    local set_o=pt['change_color']
    local stc,edc=pt['startcolor'],pt['endcolor']
    local t_color=edc-stc
    local d_c=t*t_color+stc
    local sta,eda=pt['start_alpha'],pt['end_alpha']
    local t_alpha=eda-sta
    local d_a=t*t_alpha+sta
    local angle_0=sa*(2*math.pi/360)-math.pi/2
    local angle_f=ea*(2*math.pi/360)-math.pi/2
    local t_arc=t*(angle_f-angle_0)

    -- Draw background ring
    if angle_0>angle_f then
    cairo_arc_negative(cr,xc,yc,ring_r,angle_0,angle_f)
    else
    cairo_arc(cr,xc,yc,ring_r,angle_0,angle_f)
    end
    cairo_set_source_rgba(cr,rgb_to_r_g_b(bgc,bga))
    cairo_set_line_width(cr,ring_w)
    cairo_stroke(cr)

    -- Draw indicator ring
    if angle_0>angle_f then
    cairo_arc_negative(cr,xc,yc,ring_r,angle_0,angle_0+t_arc)
    else
        cairo_arc(cr,xc,yc,ring_r,angle_0,angle_0+t_arc)
    end
    if set_o==3 then
    cairo_set_source_rgba(cr,rgb_to_r_g_b(d_c,d_a))
    else
        if set_o==2 then
        cairo_set_source_rgba(cr,rgb_to_r_g_b(fgc,d_a))

        else
            if set_o==1 then
            cairo_set_source_rgba(cr,rgb_to_r_g_b(d_c,fga))
            else
                if set_o==0 then
                cairo_set_source_rgba(cr,rgb_to_r_g_b(fgc,fga))
                else
                    if set_o==4 then
                    change_clr_alpha(cr1,t1,pt1,1)
                    else
                        if set_o==5 then
                        change_clr_alpha(cr1,t1,pt1,2)
                        else
                        change_clr_alpha(cr1,t1,pt1,3)
                        end
                    end
                end
            end
        end
    end
    --cairo_set_source_rgba(cr,rgb_to_r_g_b(fgc,fga))
    cairo_stroke(cr)
end

-- alpha变换(配合绘制圆(可变色))
function change_clr_alpha(cr,t,pt,num1)

    local w,h=conky_window.width,conky_window.height

    local xc,yc,ring_r,ring_w,sa,ea=pt['x'],pt['y'],pt['radius'],pt['thickness'],pt['start_angle'],pt['end_angle']
    local bgc, bga, fgc, fga=pt['bg_colour'], pt['bg_alpha'], pt['fg_colour'], pt['fg_alpha']
    local n1=pt['max']
    if n1<60 then
    n1=60
    end
    local set_o=pt['change_color']
    local stc,edc=pt['startcolor'],pt['endcolor']
    local t_color=tonumber(edc-stc)
    local d_c=t*t_color+stc
    local sta,eda=pt['start_alpha'],pt['end_alpha']
    local t_alpha=eda-sta
    local d_a=t*t_alpha+sta
    local angle_0=sa*(2*math.pi/360)-math.pi/2
    local angle_f=ea*(2*math.pi/360)-math.pi/2
    local t_arc=t*(angle_f-angle_0)
    cairo_new_path(cr)

    if num1==1 or num1==3 then

    if angle_0>angle_0+t_arc then
    cairo_arc_negative(cr,xc,yc,ring_r+ring_w/2,angle_0,angle_0+t_arc)
    cairo_line_to(cr,xc+(ring_r-ring_w/2)*math.cos(angle_0+t_arc),yc+(ring_r-ring_w/2)*math.sin(angle_0+t_arc))

    cairo_arc(cr,xc,yc,ring_r-ring_w/2,angle_0+t_arc,angle_0)
        cairo_line_to(cr,xc+(ring_r+ring_w/2)*math.cos(angle_0),yc+(ring_r+ring_w/2)*math.sin(angle_0))
    cairo_set_source_rgba(cr,1,1,1,0.5)
        cairo_set_line_width(cr,1)
    cairo_stroke_preserve(cr)

        cairo_stroke(cr)

    else
    cairo_arc(cr,xc,yc,ring_r+ring_w/2,angle_0,angle_0+t_arc)
    cairo_line_to(cr,xc+(ring_r-ring_w/2)*math.cos(angle_0+t_arc),yc+(ring_r-ring_w/2)*math.sin(angle_0+t_arc))

    cairo_arc_negative(cr,xc,yc,ring_r-ring_w/2,angle_0+t_arc,angle_0)
        cairo_line_to(cr,xc+(ring_r+ring_w/2)*math.cos(angle_0),yc+(ring_r+ring_w/2)*math.sin(angle_0))
    cairo_set_source_rgba(cr,1,1,1,0.5)
        cairo_set_line_width(cr,1)
    cairo_stroke_preserve(cr)

        cairo_stroke(cr)

    end

    end

    if num1==2 or num1==3 then
    cairo_new_path(cr)
    local angle_k,d_k=angle_0,stc
    local sp_angle= (angle_f-angle_0)/n1
    local f_angle=t*n1
    local r_k,g_k,b_k,a_k=rgb_to_r_g_b(d_k)
    local a_k=sta
    local r_ke,g_ke,b_ke=rgb_to_r_g_b(edc)
    local a_ke=eda
    local sp_dr,sp_dg,sp_db,sp_a=(r_ke-r_k)/n1,(g_ke-g_k)/n1,(b_ke-b_k)/n1,(a_ke-a_k)/n1

        for i=1,f_angle do
        if angle_k<angle_k+sp_angle then
              cairo_arc(cr,xc,yc,ring_r,angle_k,angle_k+sp_angle)
        else
        cairo_arc_negative(cr,xc,yc,ring_r,angle_k,angle_k+sp_angle)
        end
        cairo_set_source_rgba(cr,r_k,g_k,b_k,a_k)
        cairo_set_line_width(cr,ring_w-2)
        angle_k=angle_k+sp_angle
        r_k=r_k+sp_dr
        g_k=g_k+sp_dg
        b_k=b_k+sp_db
        a_k=a_k+sp_a
        cairo_stroke(cr)
        cairo_new_path(cr)
        end
    end
end
]]

-- 主函数
function conky_ring_stats()
    local function setup_rings(cr,pt)
        local str=''
        local value=0

        if pt['name'] == 'battery_percent' then
        	str='${if_existing /sys/class/power_supply/BAT0}${battery_percent BAT0}${else}${if_existing /sys/class/power_supply/BAT1}${battery_percent BAT1}${else}0${endif}${endif}'
        else
        	str=string.format('${%s %s}',pt['name'],pt['arg'])
        end
 
        str=conky_parse(str)
        value=tonumber(str)
        if value == nil then value = 0 end
        pct=value/pt['max']

        draw_ring(cr,pct,pt)
    end

    -- Check that Conky has been running for at least 5s

    if conky_window==nil then return end
    local cs=cairo_xlib_surface_create(conky_window.display,conky_window.drawable,conky_window.visual, conky_window.width,conky_window.height)

    local cr=cairo_create(cs)

    local updates=conky_parse('${updates}')
    update_num=tonumber(updates)

    if update_num>5 then
        for i in pairs(settings_table) do
            setup_rings(cr,settings_table[i])
        end
    end
    cairo_surface_destroy(cs)
    cairo_destroy(cr)
end
